package cn.newfeifan.mall.module.product.service.spuapply;

import cn.newfeifan.mall.module.product.controller.admin.skuapply.vo.SkuApplySaveReqVO;
import cn.newfeifan.mall.module.product.controller.admin.spu.vo.ProductSkuSaveReqVO;
import cn.newfeifan.mall.module.product.enums.spu.ProductSpuStatusEnum;
import cn.newfeifan.mall.module.product.service.sku.ProductSkuService;
import cn.newfeifan.mall.module.product.service.skuapply.SkuApplyService;
import cn.newfeifan.mall.module.system.controller.admin.user.vo.user.UserShopDetailsVO;
import cn.newfeifan.mall.module.system.service.user.AdminUserService;
import org.springframework.context.annotation.Lazy;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;

import org.springframework.validation.annotation.Validated;

import java.util.*;

import cn.newfeifan.mall.module.product.controller.admin.spuapply.vo.*;
import cn.newfeifan.mall.module.product.dal.dataobject.spuapply.SpuApplyDO;
import cn.newfeifan.mall.framework.common.pojo.PageResult;
import cn.newfeifan.mall.framework.common.util.object.BeanUtils;

import cn.newfeifan.mall.module.product.dal.mysql.spuapply.SpuApplyMapper;

import static cn.newfeifan.mall.framework.common.exception.util.ServiceExceptionUtil.exception;
import static cn.newfeifan.mall.framework.common.util.collection.CollectionUtils.getMinValue;
import static cn.newfeifan.mall.framework.common.util.collection.CollectionUtils.getSumValue;
import static cn.newfeifan.mall.module.product.enums.ErrorCodeConstants.*;

/**
 * 商品spu申请 Service 实现类
 *
 * @author 非繁人
 */
@Service
@Validated
public class SpuApplyServiceImpl implements SpuApplyService {

    @Resource
    private SpuApplyMapper spuApplyMapper;
    @Resource
    private SkuApplyService skuApplyService;
    @Resource
    @Lazy // 循环依赖，避免报错
    private ProductSkuService productSkuService;
    @Resource
    private AdminUserService adminUserService;

    @Override
    public Long createSpuApply(SpuApplySaveReqVO createReqVO) {
        // 校验 SKU
        List<SkuApplySaveReqVO> skuSaveReqList = createReqVO.getSkus();
        productSkuService.validateSkuList(BeanUtils.toBean(skuSaveReqList, ProductSkuSaveReqVO.class), createReqVO.getSpecType(), createReqVO.getHighPrecision());

        SpuApplyDO spuApply = BeanUtils.toBean(createReqVO, SpuApplyDO.class);
        // 初始化 SPU 中 SKU 相关属性
        initSpuFromSkus(spuApply, skuSaveReqList);
        // 插入 SPU
        spuApplyMapper.insert(spuApply);
        // 插入 SKU
        skuApplyService.createSkuList(spuApply.getId(), skuSaveReqList);
        // 返回
        return spuApply.getId();
    }

    /**
     * 基于 SKU 的信息，初始化 SPU 的信息
     * 主要是计数相关的字段，例如说市场价、最大最小价、库存等等
     *
     * @param spu  商品 SPU
     * @param skus 商品 SKU 数组
     */
    private void initSpuFromSkus(SpuApplyDO spu, List<SkuApplySaveReqVO> skus) {
        //是否是高精度商品
        if (!spu.getHighPrecision()) {
            // sku 单价最低的商品的价格
            spu.setPrice(getMinValue(skus, SkuApplySaveReqVO::getPrice));
            // sku 单价最低的商品的市场价格
            spu.setMarketPrice(getMinValue(skus, SkuApplySaveReqVO::getMarketPrice));
            // sku 单价最低的商品的成本价格
            spu.setCostPrice(getMinValue(skus, SkuApplySaveReqVO::getCostPrice));
        } else {
            // sku 单价最低的商品的价格
            spu.setHighPrecisionPrice(getMinValue(skus, SkuApplySaveReqVO::getHighPrecisionPrice));
        }

        // skus 库存总数
        spu.setStock(getSumValue(skus, SkuApplySaveReqVO::getStock, Integer::sum));
        // 若是 spu 已有状态则不处理
        if (spu.getStatus() == null) {
            spu.setStatus(ProductSpuStatusEnum.ENABLE.getStatus()); // 默认状态为上架
            spu.setSalesCount(0); // 默认商品销量
            spu.setBrowseCount(0); // 默认商品浏览量
        }
    }

    @Override
    public void updateSpuApply(SpuApplySaveReqVO updateReqVO) {
        // 校验存在
        validateSpuApplyExists(updateReqVO.getId());
        // 更新
        SpuApplyDO updateObj = BeanUtils.toBean(updateReqVO, SpuApplyDO.class);
        spuApplyMapper.updateById(updateObj);
    }

    @Override
    public void deleteSpuApply(Long id) {
        // 校验存在
        validateSpuApplyExists(id);
        // 删除
        spuApplyMapper.deleteById(id);
    }

    private void validateSpuApplyExists(Long id) {
        if (spuApplyMapper.selectById(id) == null) {
            throw exception(SPU_APPLY_NOT_EXISTS);
        }
    }

    @Override
    public SpuApplyDO getSpuApply(Long id) {
        return spuApplyMapper.selectById(id);
    }

    @Override
    public PageResult<SpuApplyDO> getSpuApplyPage(SpuApplyPageReqVO pageReqVO) {

        UserShopDetailsVO userShopDetails = adminUserService.getUserShopDetails();
        pageReqVO.setShopId(userShopDetails.getShopId());
        pageReqVO.setMerchantId(userShopDetails.getMerId());
        return spuApplyMapper.selectPage(pageReqVO);
    }

    @Override
    public SpuApplyDO getSpu(Long id) {
        return spuApplyMapper.selectById(id);
    }

}